==> tecfy_core_package-0.0.4.tar.gz: lib/controllers/entity/entity_list.controller.dart <==
part of tecfy_core_client_package;

class EntityListController extends BaseController{
  
  EntityService _service;
  Entity entity;
  List<dynamic> data;
  List<EntityFilter> filters;
  int pageIndex =0;
  int pageIndexMax =0;
  bool hasNextPage =true;
  ScrollController scrollController;

  AnimationController controller;
  AnimationController counterAnimationController;
  Animation<double> sizeAnimation;
  static const double size = 800.0;
  double size1 = 0;
  double size2 = 0;
  double size3 = 0;
  double size4 = 0;

  EntityListController(State<StatefulWidget> state,String entityName) : super(state){


    scrollController = ScrollController();
    scrollController.addListener(_scrollListner);

    // if(LayoutService.disableAnimation(context)){
      controller = AnimationController(
          vsync: state as TickerProviderStateMixin,
          duration: Duration(milliseconds: 600));
      sizeAnimation = Tween<double>(begin: 1.0, end: 0.0).animate(
          CurvedAnimation(parent: controller, curve: Curves.fastOutSlowIn));
      controller.addListener(() {
        size1 = size * getValueForAnimcation(sizeAnimation.value, 0.3);
        size2 = size * 2 * getValueForAnimcation(sizeAnimation.value, 0.20);
        size3 = size * 4 * getValueForAnimcation(sizeAnimation.value, 0.10);
        size4 = size * 8 * sizeAnimation.value;
        setState();
      });
    // }
    _service = EntityService();
    filters = List<EntityFilter>();
    print('-------------- EntityListController => $entityName');
    entity = EntityService.getEntity(entityName);
    loadData();
  }


  @override dispose(){
    scrollController?.removeListener(_scrollListner);
    scrollController?.dispose();
    controller?.dispose();
    counterAnimationController?.dispose();
    super.dispose();
  }

  
  _scrollListner(){

    if(scrollController.position.extentAfter < 100){
      //if(isLoadingNext)return;
      //isLoadingNext =true;
      setState();
      print('Start Loading Next--------------------------------${scrollController.position.extentAfter}');
      gotoMextPageIndex();
    }
  }

  getValueForAnimcation(double value, double th) => (value < th) ? 0: value;
  setCountdownTimerController(){
    if(controller == null) return;
    counterAnimationController = AnimationController(vsync: state as TickerProviderStateMixin, duration: Duration(seconds: 60));
    counterAnimationController.forward(); //from: counterAnimationController.value == 0.0 ? 1.0: counterAnimationController.value
  }
  refreshAnimation() {
    if(controller == null) return;
    size1 = size;
    size2 = size * 2;
    size3 = size * 3;
    size4 = size * 4;
    controller.reset();
    controller.forward();
  }


  String sortProp;
  bool sortAsc = true;

  sortBy(Property prop){
    if(prop.dbName != sortProp){
      sortProp = prop.dbName;
      sortAsc =true;
    }else sortAsc = !sortAsc;

    data.sort((a,b)=> sortAsc ? a[sortProp].toString().compareTo(b[sortProp].toString()) : b[sortProp].toString().compareTo(a[sortProp].toString()) );
    setState();
  }

  bool get hasPageing => (data?.length ?? 0) >= EntityService.pageSize;

  gotoMextPageIndex(){
    pageIndex++;
    pageIndexMax = max(pageIndexMax,pageIndex);
    setLoading();
    loadData(loadFromServer:true);
  }

  gotoPageIndex(int i){
    pageIndex=i;
    setLoading();
    loadData(loadFromServer:true);
  }

  var isLoading =false;
  loadData({bool loadFromServer=false})async{
    if(isLoading)return;
    isLoading=true;
    var result = await _service.getData(entity.dbName,filter:filters,forceFromServer: loadFromServer,pageIndex: pageIndex);
    setResume();
    if(!result.success){
      data = [];
      result.showMessage(context);
    }else{
      if(LayoutService.isMobile(context) && pageIndex > 0)
        data.addAll(result.data);
      else
        data = result.data;
      
      if(result.data.length < EntityService.pageSize)
        hasNextPage =false;
    }
    setState();
    if(pageIndex == 0) refreshAnimation();
    isLoading=false;
  }

  Future loadFromServer(){
    return loadData(loadFromServer:true);
  }

  onRefresh(){
    pageIndex =0;
    pageIndexMax =0;
    hasNextPage =true;
    setLoading();
    loadData(loadFromServer:true);
  }

  onAdd()async{
    var result = await AppNavigator.navigateTo(context, ()=>EntityFormPage(entity.dbName), animationType: AnimationTypes.slideUp);
    if(result == true){
      loadFromServer();
    }
  }

  onUpdate(d)async {
    var result = await AppNavigator.navigateTo(context, ()=>EntityFormPage(entity.dbName,d), animationType: AnimationTypes.slideUp);
    if(result == true){
      loadFromServer();
    }
  }

  onDelete(d)async{
    
    if(await confirmMessage('هل انت متاكد من الحذف؟')){
      var result = await EntityService().delete(entity.dbName,d['id']);
      if(result.success){
        showMessage('تم الحذف بنجاح', type: MessageType.info);
        loadFromServer();
      }else{
        showMessage('خطأ اثناء الحذف',type: MessageType.error);
      }
    }

  }

  lowerFirst(String s){
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  String get getPhotoDataPropName => '${entity.properties.firstWhere((d)=>d.isImage).dbName}Id';
  
}

==> tecfy_core_package-0.1.3.tar.gz: lib/controllers/entity/entity_list.controller.dart <==
part of tecfy_core_client_package;

class EntityListController extends BaseController{
  
  EntityService _service;
  Entity entity;
  List<dynamic> data;
  List<EntityFilter> filters;
  int pageIndex =0;
  int pageIndexMax =0;
  bool hasNextPage =true;
  ScrollController scrollController;

  AnimationController controller;
  AnimationController counterAnimationController;
  Animation<double> sizeAnimation;
  static const double size = 800.0;
  double size1 = 0;
  double size2 = 0;
  double size3 = 0;
  double size4 = 0;

  EntityListController(State<StatefulWidget> state,String entityName) : super(state){


    scrollController = ScrollController();
    scrollController.addListener(_scrollListner);

    // if(LayoutService.disableAnimation(context)){
      controller = AnimationController(
          vsync: state as TickerProviderStateMixin,
          duration: Duration(milliseconds: 600));
      sizeAnimation = Tween<double>(begin: 1.0, end: 0.0).animate(
          CurvedAnimation(parent: controller, curve: Curves.fastOutSlowIn));
      controller.addListener(() {
        size1 = size * getValueForAnimcation(sizeAnimation.value, 0.3);
        size2 = size * 2 * getValueForAnimcation(sizeAnimation.value, 0.20);
        size3 = size * 4 * getValueForAnimcation(sizeAnimation.value, 0.10);
        size4 = size * 8 * sizeAnimation.value;
        setState();
      });
    // }
    _service = EntityService();
    filters = List<EntityFilter>();
    print('-------------- EntityListController => $entityName');
    entity = EntityService.getEntity(entityName);
    loadData();
  }


  @override dispose(){
    scrollController?.removeListener(_scrollListner);
    scrollController?.dispose();
    controller?.dispose();
    counterAnimationController?.dispose();
    super.dispose();
  }

  
  _scrollListner(){

    if(scrollController.position.extentAfter < 100){
      //if(isLoadingNext)return;
      //isLoadingNext =true;
      setState();
      print('Start Loading Next--------------------------------${scrollController.position.extentAfter}');
      gotoMextPageIndex();
    }
  }

  getValueForAnimcation(double value, double th) => (value < th) ? 0: value;
  setCountdownTimerController(){
    if(controller == null) return;
    counterAnimationController = AnimationController(vsync: state as TickerProviderStateMixin, duration: Duration(seconds: 60));
    counterAnimationController.forward(); //from: counterAnimationController.value == 0.0 ? 1.0: counterAnimationController.value
  }
  refreshAnimation() {
    if(controller == null) return;
    size1 = size;
    size2 = size * 2;
    size3 = size * 3;
    size4 = size * 4;
    controller.reset();
    controller.forward();
  }


  String sortProp;
  bool sortAsc = true;

  sortBy(Property prop){
    if(prop.dbName != sortProp){
      sortProp = prop.dbName;
      sortAsc =true;
    }else sortAsc = !sortAsc;

    data.sort((a,b)=> sortAsc ? a[sortProp].toString().compareTo(b[sortProp].toString()) : b[sortProp].toString().compareTo(a[sortProp].toString()) );
    setState();
  }

  bool get hasPageing => (data?.length ?? 0) >= EntityService.pageSize;

  gotoMextPageIndex(){
    pageIndex++;
    pageIndexMax = max(pageIndexMax,pageIndex);
    setLoading();
    loadData(loadFromServer:true);
  }

  gotoPageIndex(int i){
    pageIndex=i;
    setLoading();
    loadData(loadFromServer:true);
  }

  var isLoading =false;
  loadData({bool loadFromServer=false})async{
    if(isLoading)return;
    isLoading=true;
    var result = await _service.getData(entity,filter:filters,forceFromServer: loadFromServer,pageIndex: pageIndex);
    setResume();
    if(!result.success){
      data = [];
      result.showMessage(context);
    }else{
      if(LayoutService.isMobile(context) && pageIndex > 0)
        data.addAll(result.data);
      else
        data = result.data;
      
      if(result.data.length < EntityService.pageSize)
        hasNextPage =false;
    }
    setState();
    if(pageIndex == 0) refreshAnimation();
    isLoading=false;
  }

  Future loadFromServer(){
    return loadData(loadFromServer:true);
  }

  onRefresh(){
    pageIndex =0;
    pageIndexMax =0;
    hasNextPage =true;
    setLoading();
    loadData(loadFromServer:true);
  }

  onAdd()async{
    var result = await AppNavigator.navigateTo(context, ()=>EntityFormPage(entity.dbName), animationType: AnimationTypes.slideUp);
    if(result == true){
      loadFromServer();
    }
  }

  onUpdate(d)async {
    var result = await AppNavigator.navigateTo(context, ()=>EntityFormPage(entity.dbName,d), animationType: AnimationTypes.slideUp);
    if(result == true){
      loadFromServer();
    }
  }

  onDelete(d)async{
    
    if(await confirmMessage('هل انت متاكد من الحذف؟')){
      var result = await EntityService().delete(entity,d['id']);
      if(result.success){
        showMessage('تم الحذف بنجاح', type: MessageType.info);
        loadFromServer();
      }else{
        showMessage('خطأ اثناء الحذف',type: MessageType.error);
      }
    }

  }

  lowerFirst(String s){
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  String get getPhotoDataPropName => '${entity.properties.firstWhere((d)=>d.isImage).dbName}Id';
  
}
